declare name 		"Synth13";
declare version 	"1.11";
declare author 		"LP";
declare license 	"BSD";
declare copyright 	"(c)CIEREC 2017";

//-----------------------------------------------
// SynthC7 
//-----------------------------------------------
import("stdfaust.lib");
import("maxmsp.lib") ;
msp = library("maxmsp.lib");
import("music.lib") ;
mus = library("music.lib");


//----------- note -------------
gate = checkbox("gate");
gate2 = gate : \(x).(x>x');
freq = hslider("freq ", 440, 20, 2000, 1);
gain = hslider("gain", 1, 0, 1, 0.01)  : si.smooth(0.999);

swbruit = checkbox ("swbruit") ;

freq1 = freq * vibrato * SampleHold ;

//-----------------------------------------------
// 			osc(freq) : Sinusoidal Oscillator
//-----------------------------------------------
xxx = environment {
	tablesize 	= 1 << 16;
	samplingfreq	= SR;

	time 		= (+(1)~_ ) - 1; 			// 0,1,2,3,...
	sinwaveform 	= float(time)*(2.0*PI)/float(tablesize) : sin;

	decimal(x)	= x - floor(x);
	phase(trig, freq) 	= freq/float(samplingfreq) : (+ : decimal : if(trig>0, 0, _)) ~ _ : *(float(tablesize));
	osc(trig, freq)	= rdtable(tablesize, sinwaveform, int(phase(trig,freq)) );
	osci(trig, freq)	= s1 + d * (s2 - s1)
			with {
				i = int(phase(trig, freq));
				d = decimal(phase(trig, freq));
				s1 = rdtable(tablesize+1,sinwaveform,i);
				s2 = rdtable(tablesize+1,sinwaveform,i+1);};
};


//**** definit un indicer qui va permettre de lire une table ****
// parametres : dur = duree en echantillons (minimun 1)
//				trig = quand > 0 declenche le cycle
// sortie : valeur variant de 0 a 1.0 en dur echantillons, reste a 1.0 ensuite
// NOTE : la fonction est protegee contre les durees nulles et les durees qui changent en cours de route

indicer(dur, trig) = I ~ (_,_) : /
	with {
			I(p, d) = ba.if(trig>0.0, 0.0, min(d, p+1.0)),
					  max(1.0, ba.if(trig>0.0, dur, d));
	};
	
ifunctTab(bpf, low, high, dur, trig) = s1 * (1.0 - d) + s2 *d : *(high - low) + low 
 	with {
 			phase = 100.0 * indicer(dur, trig) ;
			i = int(phase) ;
 			d = phase-int(phase);
 			s1 = rdtable(101, bpf(time), i);
 			s2 = rdtable(101, bpf(time), i+1);
		};

bpf9  = mus.bpf.start(0,1.0) : mus.bpf.point(25,0.25) : mus.bpf.point(50,0.1) : mus.bpf.point(75,0.05) : mus.bpf.end(100,0);
bpf10 = mus.bpf.start(0,0.5) : mus.bpf.point(20,1.0)  : mus.bpf.point(50,0.1) : mus.bpf.point(75,0.02) : mus.bpf.end(100,0);
bpf11 = mus.bpf.start(0,0.0) : mus.bpf.point(10,0.02) : mus.bpf.point(15,0.1) : mus.bpf.point(25,0.5)  : mus.bpf.point(30,0.95) : mus.bpf.point(33,1) : mus.bpf.point(85,0.95) : mus.bpf.point(90,0.9) : mus.bpf.point(95,0.05) : mus.bpf.end(100,0);
bpf12 = mus.bpf.start(0,0.0) : mus.bpf.point(10,0.2)  : mus.bpf.point(15,0.10): mus.bpf.point(25,0.5)  : mus.bpf.point(30,0.95) : mus.bpf.point(33,1.0) : mus.bpf.end(100,0);
bpf13 = mus.bpf.start(0,0.0) : mus.bpf.point(13,0.5)  : mus.bpf.point(30,1.0) : mus.bpf.point(60,0.2)  : mus.bpf.point(75,0.04) : mus.bpf.end(100,0);
bpf14 = mus.bpf.start(0,1.0) : mus.bpf.point(12,0.75) : mus.bpf.point(25,1.0) : mus.bpf.point(50,0.25) : mus.bpf.point(75,0.05) : mus.bpf.end(100,0);
bpf15 = mus.bpf.start(0,1.0) : mus.bpf.point(5,0.1)   : mus.bpf.point(10,0.02) : mus.bpf.point(25,0.0) : mus.bpf.end(100,0);
bpf16 = mus.bpf.start(0,0.0) : mus.bpf.point(15,0.1) : mus.bpf.point(30,0.50) : mus.bpf.point(40,0.95) : mus.bpf.point(50,1.0) : mus.bpf.point(60,0.96) : mus.bpf.point(70,0.5) : mus.bpf.point(85,0.1) : mus.bpf.end(100,0);
bpf17 = mus.bpf.start(0,1.0) : mus.bpf.point(11,0.5) : mus.bpf.point(25,0.25) : mus.bpf.point(50,0.1) : mus.bpf.point(75,0.05) : mus.bpf.end(100,0);


env(trig, dur, bpfxf, low, high) =  
	ba.if (bpfx == 9,  ifunctTab(bpf9,  low, high, dur, trig),
	ba.if (bpfx == 10, ifunctTab(bpf10, low, high, dur, trig),
	ba.if (bpfx == 11, ifunctTab(bpf11, low, high, dur, trig),
	ba.if (bpfx == 12, ifunctTab(bpf12, low, high, dur, trig),
	ba.if (bpfx == 13, ifunctTab(bpf13, low, high, dur, trig),
	ba.if (bpfx == 14, ifunctTab(bpf14, low, high, dur, trig),
	ba.if (bpfx == 15, ifunctTab(bpf15, low, high, dur, trig),
	ba.if (bpfx == 16, ifunctTab(bpf16, low, high, dur, trig),
					ifunctTab(bpf17, low, high, dur, trig)
	)))))))) with { bpfx = int(bpfxf); };

swenv = checkbox("[8] swenv");
indexFM = hslider("[9]index [multi:1]", 6, 0, 20, 0.1) : si.smooth(0.999);	
dur = 3000 * attack / 100. ;
nbpf = 9;
envelop(swenv) = select3(swenv, env1, env2);

env1 = env(gate2, nsamples, nbpf, 0, 1)
	with { 
		nsamples = max(10, dur * ma.SR / 1000.0) ; 
	}: si.smooth(0.9);
	
	
//------------ enveloppe tenue --------
	
attack = hslider("attaque", 100, 1, 2000, 1) ;
release = attack * 2 ;
	
durr = select2(gate, release, attack);
env2= gate:si.smooth(0.999);	
				

				
//----------- oscillos -------
	
fmfm2 (freq, amp, transpfm, index, transp) = 
	env2 * amp *
	osci(freq * transp + 
		env2 * amp * index *
			freq * transpfm * transp * osci(freq * transpfm * transp)) ;
			
fmfm2 (freq, amp, transpfm, index, transp) = 
	env2 * amp *
	os.osci(freq * transp + 
		env2 * amp * index *
			freq * transpfm * transp * os.osci(freq * transpfm * transp)) ;




fmfm1 (trig, freq, amp, transpfm, index, transp) = 
 	env1 * amp *
 	xxx.osci(trig, freq * transp + 
 		env1 * amp * index *
 			freq * transpfm * transp * xxx.osci(trig, freq * transpfm * transp)) ;

synthfm(swenv) = select2(swenv, fmfm1 (gate2, freq1, gain, fmfreq, indexFM, 1.0) , 
 	fmfm2 (freq1, gain,  fmfreq, indexFM, 1.0));

noiseenv(swenv) = select2(swenv, noise * env1 * gain , noise * env2 * gain);

// synthfm(swenv) = fmfm2 (freq, gain, fmfreq, fmfreq, 1);

//--------- synth -----------
freqmod = hslider("v:General Parameters/freqmod", 1, 1, 4, 1);
fmfreq = sqrt(freqmod):+(0.001);

synth = select2(swbruit, synthfm(swenv), noiseenv(swenv));
// synth = synthfm(swenv);
	
//-------------- vibrato ---------------//
ampvib = hslider("ampvib", 0.01, 0, 0.1, 0.001);
freqvib = 5.6;

vibrato =  1 + (ampvib * os.osci(freqvib));
	
//------------- sample&hold -----------//	
cocoX  = hslider("h:SHold/cocoX [style:knob]", 400, 1 , 1000, 0.1);
cocoY = hslider("h:SHold/cocoY [style:knob]", 10, 1, 50, 0.1);
ampSH = hslider("h:SHold/ampSH [style:knob]", 0.1, 0, 0.9, 0.01) ;

trigSH  = pulse(SR/cocoY) ;
SampAndHold (x, trig) = (* (1 - trig) + x * trig) ~ _ ;	
SampleHold = 1 + (ampSH * SampAndHold(os.triangle(cocoX), trigSH));

//------------- filtres -----------//	
f1 = hslider("v:filtre/f1 [multi:0]" , 500, 20, 4000, 0.01) : si.smooth(0.999);

// f1f = f1 * freq / 440.;
f1f = select2(swenv,env1, env2):*(f1 * freq / 440.):+(50): si.smooth(0.999);
f2 = 2 * f1f ;
gain01 = 0,8;
gain02 = 0.8;
Q = 1;

filtre1(x) = msp.BPF(x, f1f, gain01, Q) ;
filtre2(x) = msp.BPF(x, f2, gain02, Q) ;

filtres(x) = filtre1(x), filtre2(x) ;

//------------- delay -----------//	
delmax = 65536;
mixdel = hslider("mixecho", 0.2, 0, 1, 0.01) ;
mixwet = 1.0 - mixdel;
deldur1 = 0.250 * ma.SR ;
deldur2 = 0.180 * ma.SR ;
echos = _<:((_,_:> de.delay (delmax, deldur1)) ~ *(0.3)), ((_,_:> de.delay (delmax, deldur2)) ~ *(0.4));

tbar		= 0.1;					// attack/release time in seconds
gbar		= exp(-1/(ma.SR*tbar));	// corresponding gain factor
envbar		= abs : *(1-gbar) : + ~ *(gbar) : ba.linear2db;
	
left_meter(x)	= attach(x, envbar(x) : hbargraph("left", -96, 10));	
right_meter(x)	= attach(x, envbar(x) : hbargraph("right", -96, 10));	
//------------- process -----------//	
process = vgroup("Synth1 [style:multikeyboard]", synth : filtres <: *(mixdel), *(mixwet) :  echos, (_<:_,_) :> (_,_):vgroup("bars",(left_meter,right_meter))); 